@@ -9,7 +9,7 @@ from pulpcore.plugin.serializers import (
     DetailRelatedField,
     ContentChecksumSerializer,
     ModelSerializer,
-    NoArtifactContentSerializer,
+    NoArtifactContentUploadSerializer,
     RelatedField,
     RemoteSerializer,
     RepositorySerializer,
@@ -38,6 +38,7 @@ from .models import (
 )
 from pulp_ansible.app.schema import COPY_CONFIG_SCHEMA
 from pulp_ansible.app.tasks.utils import parse_collections_requirements_file
+from pulp_ansible.app.tasks.signature import verify_signature_upload
 
 
 class RoleSerializer(SingleArtifactContentSerializer):
@@ -122,9 +123,16 @@ class AnsibleRepositorySerializer(RepositorySerializer):
     last_synced_metadata_time = serializers.DateTimeField(
         help_text=_("Last synced metadata time."), allow_null=True, required=False
     )
+    keyring = serializers.FilePathField(
+        path=settings.ANSIBLE_CERTS_DIR,
+        help_text=_("Location of keyring used to verify signatures uploaded to this repository"),
+        allow_blank=True,
+        default="",
+        required=False,
+    )
 
     class Meta:
-        fields = RepositorySerializer.Meta.fields + ("last_synced_metadata_time",)
+        fields = RepositorySerializer.Meta.fields + ("last_synced_metadata_time", "keyring")
         model = AnsibleRepository
 
 
@@ -501,7 +509,7 @@ class CollectionVersionSerializer(SingleArtifactContentSerializer, ContentChecks
         model = CollectionVersion
 
 
-class CollectionVersionSignatureSerializer(NoArtifactContentSerializer):
+class CollectionVersionSignatureSerializer(NoArtifactContentUploadSerializer):
     """
     A serializer for signature models.
     """
@@ -511,17 +519,38 @@ class CollectionVersionSignatureSerializer(NoArtifactContentSerializer):
         view_name_pattern=r"content(-.*/.*)-detail",
         queryset=CollectionVersion.objects.all(),
     )
-    pubkey_fingerprint = serializers.CharField(help_text=_("The fingerprint of the public key."))
+    pubkey_fingerprint = serializers.CharField(
+        help_text=_("The fingerprint of the public key."),
+        read_only=True,
+    )
     signing_service = RelatedField(
         help_text=_("The signing service used to create the signature."),
         view_name="signing-services-detail",
-        queryset=SigningService.objects.all(),
+        read_only=True,
         allow_null=True,
     )
 
+    def __init__(self, *args, **kwargs):
+        """Ensure `file` field is required."""
+        super().__init__(*args, **kwargs)
+        self.fields["file"].required = True
+
+    def validate(self, data):
+        """
+        Verify the signature is valid before creating it.
+        """
+        data = super().validate(data)
+
+        if "request" not in self.context:
+            # Validate is called twice, first on the viewset, and second on the create task
+            # data should be set up properly on the second time, when request isn't in context
+            data = verify_signature_upload(data)
+
+        return data
+
     class Meta:
         model = CollectionVersionSignature
-        fields = NoArtifactContentSerializer.Meta.fields + (
+        fields = NoArtifactContentUploadSerializer.Meta.fields + (
             "signed_collection",
             "pubkey_fingerprint",
             "signing_service",
