@@ -2,7 +2,9 @@
 from dynaconf import Dynaconf
 from functools import partial
 from time import sleep
+import json
 import os
+import subprocess
 import unittest
 
 from pulp_smash import api, cli, config, selectors, utils
@@ -30,6 +32,7 @@ from pulpcore.client.pulpcore import (
     ApiClient as CoreApiClient,
     TasksApi,
     SigningServicesApi,
+    StatusApi,
 )
 from pulpcore.client.pulp_ansible import (
     ApiClient as AnsibleApiClient,
@@ -42,13 +45,31 @@ from pulpcore.client.pulp_ansible import (
     RemotesGitApi,
     RemotesRoleApi,
     AnsibleRepositorySyncURL,
+    PulpAnsibleArtifactsCollectionsV3Api,
+    RepositoriesAnsibleVersionsApi,
 )
 
+from orionutils.generator import build_collection, randstr
+
 
 cfg = config.get_config()
 configuration = cfg.get_bindings_config()
 
 
+def is_galaxy_ng_installed():
+    """Returns whether or not the galaxy_ng plugin is installed."""
+    configuration = cfg.get_bindings_config()
+    core_client = CoreApiClient(configuration)
+    status_client = StatusApi(core_client)
+
+    status = status_client.status_read()
+
+    for plugin in status.versions:
+        if plugin.component == "galaxy":
+            return True
+    return False
+
+
 def set_up_module():
     """Skip tests Pulp 3 isn't under test or if pulp_ansible isn't installed."""
     require_pulp_3(unittest.SkipTest)
@@ -60,11 +81,15 @@ def gen_ansible_client():
     return AnsibleApiClient(configuration)
 
 
-def gen_ansible_remote(url=ANSIBLE_FIXTURE_URL, **kwargs):
+def gen_ansible_remote(url=ANSIBLE_FIXTURE_URL, include_pulp_auth=False, **kwargs):
     """Return a semi-random dict for use in creating a ansible Remote.
 
     :param url: The URL of an external content source.
     """
+    if include_pulp_auth:
+        kwargs["username"] = cfg.pulp_auth[0]
+        kwargs["password"] = cfg.pulp_auth[1]
+
     if "rate_limit" not in kwargs:
         kwargs["rate_limit"] = 5
 
@@ -143,6 +168,52 @@ def wait_tasks():
         running_tasks = tasks.list(state="running")
 
 
+def gen_collection_in_distribution(
+    base_path, name=None, namespace=None, versions=None, **cfg_kwargs
+):
+    """
+    Generate a randomized collection and upload it to the specified repository.
+
+    Params:
+        - base_path: distribtion base path for the repository to upload the
+          collection to.
+        - name: Name of the collection. If none is provided a random name will
+          be generated.
+        - namespace: Namespace of the collection. If none is provided a random
+          name will be generated.
+        - versions: A list of versions to create for the collection. If none
+          are specified the collection will be generated with version 1.0.0
+
+        Additional galaxy.yaml configuration options can be added as kwargs.
+        For example dependencies={"foo.bar": "1.0.0"} will set foo.bar as a
+        dependency for the newly generated collection
+
+    Returns: a dictionary with the name and namespace names of the newly generated
+    collection.
+    """
+    if versions is None:
+        versions = ["1.0.0"]
+    namespace = namespace or randstr()
+    name = name or randstr()
+
+    ansible_client = gen_ansible_client()
+    upload = PulpAnsibleArtifactsCollectionsV3Api(ansible_client)
+
+    for version in versions:
+        artifact = build_collection(
+            "skeleton",
+            config={"namespace": namespace, "name": name, "version": version, **cfg_kwargs},
+        )
+
+        upload.create(path=base_path, file=artifact.filename)
+
+        wait_tasks()
+
+        os.remove(artifact.filename)
+
+    return {"name": name, "namespace": namespace}
+
+
 class TestCaseUsingBindings(PulpTestCase):
     """A parent TestCase that instantiates the various bindings used throughout tests."""
 
@@ -151,6 +222,7 @@ class TestCaseUsingBindings(PulpTestCase):
         """Create class-wide variables."""
         cls.client = gen_ansible_client()
         cls.repo_api = RepositoriesAnsibleApi(cls.client)
+        cls.repo_version_api = RepositoriesAnsibleVersionsApi(cls.client)
         cls.remote_collection_api = RemotesCollectionApi(cls.client)
         cls.remote_git_api = RemotesGitApi(cls.client)
         cls.remote_role_api = RemotesRoleApi(cls.client)
@@ -273,6 +345,33 @@ def get_psql_smash_cmd(sql_statement):
     return ("psql", "-c", sql_statement, f"postgresql://{user}:{password}@{host}/{dbname}")
 
 
+def run_signing_script(filename):
+    """
+    Runs the test signing script manually on test calling machine.
+
+    Returns the signature file produced.
+    """
+    file = os.path.realpath(f"{__file__}/../../assets/sign-metadata.sh")
+    script = os.environ.get("TEST_PULP_SIGNING_SCRIPT", file)
+    key_id = os.environ.get("TEST_PULP_SIGNING_KEY_ID", "Pulp QE")
+    env = {"PULP_SIGNING_KEY_FINGERPRINT": key_id}
+    completed_process = subprocess.run(
+        [script, filename],
+        env=env,
+        stdout=subprocess.PIPE,
+        stderr=subprocess.PIPE,
+    )
+    if completed_process.returncode != 0:
+        raise RuntimeError(str(completed_process.stderr))
+
+    try:
+        return_value = json.loads(completed_process.stdout)
+    except json.JSONDecodeError:
+        raise RuntimeError("The signing script did not return valid JSON!")
+
+    return return_value
+
+
 def create_signing_service():
     """
     Generates an AsciiArmoredSigningService using the signing script in /assets.
@@ -309,3 +408,23 @@ def delete_signing_service(name):
     )
     cli_client = cli.Client(cfg)
     utils.execute_pulpcore_python(cli_client, "\n".join(python_cmd))
+
+
+def get_client_keyring():
+    """
+    Finds the GPG keyring for the client used in the tests.
+
+    Uses the environment variables to change the default keyring
+    TEST_PULP_CLIENT_KEYRING - The location for the GPG keyring
+    TEST_PULP_SIGNING_KEY_ID - The signing key id used by the signing script
+    """
+    keyring = os.environ.get("TEST_PULP_CLIENT_KEYRING", "~/.gnupg/pubring.kbx")
+    key_id = os.environ.get("TEST_PULP_SIGNING_KEY_ID", "Pulp QE")
+
+    # Check that key_id is in the keyring
+    cmd = ("gpg", "--list-keys", "--keyring", keyring, "--no-default-keyring")
+    stdout = subprocess.run(cmd, capture_output=True, check=True).stdout
+    if key_id not in stdout.decode():
+        raise Exception(f"Key ID: {key_id} not found in keyring: {keyring}")
+
+    return keyring
